﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Example03_PointAnimationUsingPath
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        // コンストラクター
        public MainWindow()
        {
            InitializeComponent();
            InitializeWindow();
        }

        private const double TIME = 6;
        private AnimationClock _clock;

        // [Avi]ボタンクリックイベント
        private void btnAvi_Click(object sender, RoutedEventArgs e)
        {
            var filepath = GetAviFilePath();
            if (0 < filepath.Length)
            {
                SaveAsAvi(filepath);
                MessageBox.Show("OK");
            }
        }

        // [開始]ボタンクリックイベント
        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            _clock.Controller.Begin();
        }

        // [停止]ボタンクリックイベント
        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            _clock.Controller.Stop();
        }

        
        // 円の通路用PathGeometry生成
        private PathGeometry GetPathGeometry()
        {
            var ptA = new Point(60, 140);
            var ptB = new Point(260, 140);
            var ptC = new Point(460, 140);
            
            var fig1 = new PathFigure();

            fig1.StartPoint = ptA;
            fig1.Segments.Add(new ArcSegment(ptB, new Size(50, 50), 180, false, 
                                        SweepDirection.Clockwise, true));
            fig1.Segments.Add(new ArcSegment(ptC, new Size(50, 50), 180, false, 
                                        SweepDirection.Counterclockwise, true));
            fig1.Segments.Add(new ArcSegment(ptB, new Size(50, 50), 180, false, 
                                        SweepDirection.Counterclockwise, true));
            fig1.Segments.Add(new ArcSegment(ptA, new Size(50, 50), 180, false, 
                                        SweepDirection.Clockwise, true));

            var pg = new PathGeometry();
            pg.Figures.Add(fig1);

            return pg;
        }

        // Aviファイル保存パス取得
        private string GetAviFilePath()
        {
            var dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.Filter = "aviファイル|*.avi;*.avi";
            var path = String.Empty;

            if (dlg.ShowDialog() == true)
            {
                path = dlg.FileName;
            }

            return path;
        }

        // 円の通路描画
        private void DrawPath(PathGeometry pg)
        {
            var path = new Path();
            path.Stroke = Brushes.Blue;
            path.StrokeThickness = 2;
            path.Data = pg;
            cvs1.Children.Add(path);
        }
        
        // 円とその通路の設定
        private void InitializeWindow()
        {
            const double RADIUS = 20;

            var pg = GetPathGeometry();     // 円の通路用PathGeometry生成
            DrawPath(pg);                   // 円の通路描画

            // 円の生成(EllipseGeometry使ってPathとして生成する)
            var circle = new Path();
            var gscollect = new GradientStopCollection();
            gscollect.Add(new GradientStop(Colors.White, 0));
            gscollect.Add(new GradientStop(Colors.Red, 1));
            circle.Fill = new RadialGradientBrush(gscollect);
            var eg = new EllipseGeometry(new Point(60, 140), RADIUS, RADIUS);
            circle.Data = eg;
            cvs1.Children.Add(circle);

            // EllipseGeometryへのPointAnimationUsingPath設定
            pg.Freeze();
            var pa = new PointAnimationUsingPath();
            pa.PathGeometry = pg;
            pa.Duration = TimeSpan.FromSeconds(TIME);
            pa.RepeatBehavior = RepeatBehavior.Forever;
            _clock = pa.CreateClock();
            eg.ApplyAnimationClock(EllipseGeometry.CenterProperty, _clock);
            _clock.Controller.Stop();
        }

        // Aviファイル作成
        private void SaveAsAvi(string filepath)
        {
            var fps = 48;                           // フレームレート
            var totalFrames = (int)(fps * TIME);    // 全フレーム数
            var secs = Enumerable.Range(0, totalFrames).Select(t => (((double)t) / fps));
            var aviManager = new AviFile.AviManager(filepath, false);
            AviFile.VideoStream aviStream = null;
            var tempFile = "frame.png";

            foreach (var sec in secs)
            {
                _clock.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                        TimeSeekOrigin.BeginTime);
                cvs1.UpdateLayout();
                cvs1.SaveImage(tempFile);
                var bmp = new System.Drawing.Bitmap(tempFile);
                if (aviStream == null)
                    aviStream = aviManager.AddVideoStream(true, fps, bmp);
                else
                    aviStream.AddFrame(bmp);
                bmp.Dispose();
            }
            aviManager.Close();
            _clock.Controller.Stop();
            System.IO.File.Delete(tempFile);
        }

    }

    public static class CanvasExtension
    {
        /// <summary> [SaveImage]：Canvasの画像をファイルに保存するCanvasクラスの拡張メソッド 
        /// 出典：http://qiita.com/hugo-sb/items/894914f6bbe224a45d49
        /// </summary>
        /// <param name="canvas"></param>
        /// <param name="path">保存する画像ファイル名</param>
        /// <param name="encoder">BitmapEncoderが指定されなかった場合はPNG形式になる</param>
        public static void SaveImage(this Canvas canvas, string path, BitmapEncoder encoder = null)
        {
            // レイアウトを再計算させる
            var size = new Size(canvas.ActualWidth, canvas.ActualHeight);
            canvas.Measure(size);
            canvas.Arrange(new Rect(size));

            // VisualObjectをBitmapに変換する
            var renderBitmap = new RenderTargetBitmap((int)size.Width,       // 画像の幅
                                                      (int)size.Height,      // 画像の高さ
                                                      96.0d,                 // 横96.0DPI
                                                      96.0d,                 // 縦96.0DPI
                                                      PixelFormats.Pbgra32); // 32bit(RGBA各8bit)
            renderBitmap.Render(canvas);

            // 出力用の FileStream を作成する
            using (var stream = new System.IO.FileStream(path, System.IO.FileMode.Create))
            {
                // 変換したBitmapをエンコードしてFileStreamに保存する。
                // BitmapEncoder が指定されなかった場合は、PNG形式とする。
                encoder = encoder ?? new PngBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(renderBitmap));
                encoder.Save(stream);
            }
        }

    }

}
